@@ -1,38 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
-**
-** This file is part of the QtSerialBus module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL3$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2017 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qmodbusdevice.h"
 #include "qmodbusdevice_p.h"
@@ -95,11 +62,6 @@ QModbusDevice::~QModbusDevice()
     \value NetworkPortParameter      This parameter holds the network port. \c int
     \value NetworkAddressParameter   This parameter holds the host address for network
                                      communication. \c QString
-
-    User options:
-
-    \value UserParameter             The first parameter that can be used for user-specific
-                                     purposes. \c QVariant
 */
 
 /*!
@@ -120,10 +82,11 @@ QModbusDevice::~QModbusDevice()
 
     \sa ConnectionParameter
 */
-QVariant QModbusDevice::connectionParameter(int parameter) const
+QVariant QModbusDevice::connectionParameter(ConnectionParameter parameter) const
 {
     Q_D(const QModbusDevice);
     switch (parameter) {
+#if QT_CONFIG(modbus_serialport)
     case SerialPortNameParameter:
         return d->m_comPort;
     case SerialDataBitsParameter:
@@ -134,6 +97,7 @@ QVariant QModbusDevice::connectionParameter(int parameter) const
         return d->m_stopBits;
     case SerialBaudRateParameter:
         return d->m_baudRate;
+#endif
     case NetworkPortParameter:
         return d->m_networkPort;
     case NetworkAddressParameter:
@@ -141,7 +105,7 @@ QVariant QModbusDevice::connectionParameter(int parameter) const
     default:
         break;
     }
-    return d->m_userConnectionParams.value(parameter);
+    return {};
 }
 
 /*!
@@ -152,10 +116,11 @@ QVariant QModbusDevice::connectionParameter(int parameter) const
     \sa ConnectionParameter
     \sa connectionParameter()
 */
-void QModbusDevice::setConnectionParameter(int parameter, const QVariant &value)
+void QModbusDevice::setConnectionParameter(ConnectionParameter parameter, const QVariant &value)
 {
     Q_D(QModbusDevice);
     switch (parameter) {
+#if QT_CONFIG(modbus_serialport)
     case SerialPortNameParameter:
         d->m_comPort = value.toString();
         break;
@@ -171,6 +136,7 @@ void QModbusDevice::setConnectionParameter(int parameter, const QVariant &value)
     case SerialBaudRateParameter:
         d->m_baudRate = QSerialPort::BaudRate(value.toInt());
         break;
+#endif
     case NetworkPortParameter:
         d->m_networkPort = value.toInt();
         break;
@@ -178,7 +144,7 @@ void QModbusDevice::setConnectionParameter(int parameter, const QVariant &value)
         d->m_networkAddress = value.toString();
         break;
     default:
-        d->m_userConnectionParams.insert(parameter, value);
+        Q_ASSERT_X(false, "", "Connection parameter not supported.");
         break;
     }
 }
@@ -200,6 +166,14 @@ void QModbusDevice::setConnectionParameter(int parameter, const QVariant &value)
     \value ReplyAbortedError    The reply was aborted due to a disconnection of
                                 the device.
     \value UnknownError         An unknown error occurred.
+    \value [since 6.4] InvalidResponseError An error occurred while parsing the
+                                response, or the \l {QModbusPdu::}{FunctionCode}
+                                is not supported by the current implementation.
+                                In the latter case custom Modbus client
+                                implementation can override the
+                                \l {QModbusClient::}{processResponse()} and
+                                \l {QModbusClient::}{processPrivateResponse()}
+                                methods to provide support for needed functions.
 */
 
 /*!
@@ -212,6 +186,27 @@ void QModbusDevice::setConnectionParameter(int parameter, const QVariant &value)
     \value ClosingState     The device is being closed.
 */
 
+/*!
+    \since 6.0
+    \enum QModbusDevice::IntermediateError
+
+    This enum describes possible errors that can happen during a full send and
+    receive cycle for a Modbus reply.
+
+    \value ResponseCrcError         A Modbus response with a wrong CRC was received.
+    \value ResponseRequestMismatch  A Modbus response was received but did not
+                                    match the open request, probably due to the
+                                    PDU's function code not matching.
+
+    If any of the above intermediate errors occurred, the frame is likely
+    present until the maximum number of retries has been reached.
+
+    The list of intermediate errors can be inspected from the \l QModbusReply
+    intermediate errors function.
+
+    \sa QModbusClient::numberOfRetries(), QModbusReply::intermediateErrors()
+*/
+
 /*!
     \fn QModbusDevice::errorOccurred(QModbusDevice::Error error)
 
@@ -262,7 +257,7 @@ bool QModbusDevice::connectDevice()
 */
 void QModbusDevice::disconnectDevice()
 {
-    if (state() == QModbusDevice::UnconnectedState)
+    if (auto s = state(); s == QModbusDevice::UnconnectedState || s == QModbusDevice::ClosingState)
         return;
 
     setState(QModbusDevice::ClosingState);
@@ -332,6 +327,20 @@ QString QModbusDevice::errorString() const
     return d_func()->errorString;
 }
 
+/*!
+    \since 5.14
+
+    Returns the underlying \l QIODevice used for ModBus communication or
+    \c nullptr if the device was not yet fully initialized.
+
+    \note Do not store a pointer to the underlying device, because it can be
+    invalidated at any point in time.
+*/
+QIODevice *QModbusDevice::device() const
+{
+    return d_func()->device();
+}
+
 /*!
     \fn bool QModbusDevice::open()
 
